@@ -82,6 +82,7 @@ static void rb_mtp_source_get_property (GObject *object,
 static void impl_delete (RBSource *asource);
 static RBTrackTransferBatch *impl_paste (RBSource *asource, GList *entries);
 static gboolean impl_show_popup (RBDisplayPage *page);
+static gboolean impl_uri_is_source (RBSource *asource, const char *uri);
 
 static gboolean impl_track_added (RBTransferTarget *target,
 				  RhythmDBEntry *entry,
@@ -100,6 +101,7 @@ static char *impl_build_dest_uri (RBTransferTarget *target,
 static void impl_eject (RBDeviceSource *source);
 static gboolean impl_can_eject (RBDeviceSource *source);
 
+static void impl_selected (RBDisplayPage *page);
 static void mtp_device_open_cb (LIBMTP_mtpdevice_t *device, RBMtpSource *source);
 static void mtp_tracklist_cb (LIBMTP_track_t *tracks, RBMtpSource *source);
 static RhythmDB * get_db_for_source (RBMtpSource *source);
@@ -132,6 +134,7 @@ static GMount *find_mount_for_device (GUdevDevice *device);
 
 typedef struct
 {
+	gboolean tried_open;
 	RBMtpThread *device_thread;
 	LIBMTP_raw_device_t raw_device;
 	GHashTable *entry_map;
@@ -190,6 +193,7 @@ rb_mtp_source_class_init (RBMtpSourceClass *klass)
 	object_class->get_property = rb_mtp_source_get_property;
 
 	page_class->show_popup = impl_show_popup;
+	page_class->selected = impl_selected;
 
 	source_class->impl_can_rename = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_true_function;
@@ -199,6 +203,7 @@ rb_mtp_source_class_init (RBMtpSourceClass *klass)
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_delete = impl_delete;
 	source_class->impl_paste = impl_paste;
+	source_class->impl_uri_is_source = impl_uri_is_source;
 
 	mps_class->impl_get_entries = impl_get_entries;
 	mps_class->impl_get_capacity = impl_get_capacity;
@@ -327,24 +332,16 @@ unmount_done_cb (GObject *object, GAsyncResult *result, gpointer psource)
 #endif
 
 static void
-rb_mtp_source_constructed (GObject *object)
+impl_selected (RBDisplayPage *page)
 {
-	RBMtpSource *source;
-	RBMtpSourcePrivate *priv;
-	RBEntryView *tracks;
-	RBShell *shell;
-	RBShellPlayer *shell_player;
-	GObject *player_backend;
-	GtkIconTheme *theme;
-	GdkPixbuf *pixbuf;
+	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (page);
 #if defined(HAVE_GUDEV)
 	GMount *mount;
 #endif
-	gint size;
 
-	RB_CHAIN_GOBJECT_METHOD (rb_mtp_source_parent_class, constructed, object);
-	source = RB_MTP_SOURCE (object);
-	priv = MTP_SOURCE_GET_PRIVATE (source);
+	if (priv->tried_open)
+		return;
+	priv->tried_open = TRUE;
 
 	/* try to open the device.  if gvfs has mounted it, unmount it first */
 #if defined(HAVE_GUDEV)
@@ -356,11 +353,31 @@ rb_mtp_source_constructed (GObject *object)
 						NULL,
 						NULL,
 						unmount_done_cb,
-						g_object_ref (source));
+						g_object_ref (page));
 		/* mount gets unreffed in callback */
-	} else
+	} else {
+		rb_debug ("device isn't mounted");
+		open_device (RB_MTP_SOURCE (page));
+	}
+#else
+	open_device (RB_MTP_SOURCE (page));
 #endif
-	open_device (source);
+}
+
+static void
+rb_mtp_source_constructed (GObject *object)
+{
+	RBMtpSource *source;
+	RBEntryView *tracks;
+	RBShell *shell;
+	RBShellPlayer *shell_player;
+	GObject *player_backend;
+	GtkIconTheme *theme;
+	GdkPixbuf *pixbuf;
+	gint size;
+
+	RB_CHAIN_GOBJECT_METHOD (rb_mtp_source_parent_class, constructed, object);
+	source = RB_MTP_SOURCE (object);
 
 	tracks = rb_source_get_entry_view (RB_SOURCE (source));
 	rb_entry_view_append_column (tracks, RB_ENTRY_VIEW_COL_RATING, FALSE);
@@ -397,9 +414,6 @@ rb_mtp_source_constructed (GObject *object)
 	g_object_set (source, "pixbuf", pixbuf, NULL);
 	g_object_unref (pixbuf);
 
-	if (priv->album_art_supported) {
-		priv->art_store = rb_ext_db_new ("album-art");
-	}
 }
 
 static void
@@ -587,6 +601,7 @@ rb_mtp_source_new (RBShell *shell,
 #endif
 					      "settings", g_settings_get_child (settings, "source"),
 					      "toolbar-path", "/MTPSourceToolBar",
+					      "name", _("Media Player"),
 					      NULL));
 	g_object_unref (settings);
 
@@ -842,6 +857,10 @@ device_opened_idle (DeviceOpenedData *data)
 		}
 	}
 
+	if (priv->album_art_supported) {
+		priv->art_store = rb_ext_db_new ("album-art");
+	}
+
 	target = gst_encoding_target_new ("mtpdevice", "device", "", profiles);
 	g_object_set (data->source, "encoding-target", target, NULL);
 
@@ -1026,6 +1045,24 @@ impl_delete (RBSource *source)
 	rb_list_destroy_free (sel, (GDestroyNotify) rhythmdb_entry_unref);
 }
 
+static gboolean
+impl_uri_is_source (RBSource *source, const char *uri)
+{
+	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (source);
+	char *source_uri;
+	gboolean result;
+
+	if (g_str_has_prefix (uri, "gphoto2://") == FALSE)
+		return FALSE;
+
+	source_uri = g_strdup_printf ("gphoto2://[usb:%03d,%03d]/", 
+				      priv->raw_device.bus_location,
+				      priv->raw_device.devnum);
+	result = g_str_has_prefix (uri, source_uri);
+	g_free (source_uri);
+	return result;
+}
+
 static RBTrackTransferBatch *
 impl_paste (RBSource *source, GList *entries)
 {
@@ -1078,6 +1115,7 @@ impl_track_added (RBTransferTarget *target,
 {
 	LIBMTP_track_t *track = NULL;
 	RBMtpSourcePrivate *priv = MTP_SOURCE_GET_PRIVATE (target);
+	RhythmDB *db;
 
 	track = g_hash_table_lookup (priv->track_transfer_map, dest);
 	if (track == NULL) {
@@ -1103,6 +1141,11 @@ impl_track_added (RBTransferTarget *target,
 			rb_ext_db_key_free (key);
 		}
 	}
+
+	db = get_db_for_source (RB_MTP_SOURCE (target));
+	add_mtp_track_to_db (RB_MTP_SOURCE (target), db, track);
+	g_object_unref (db);
+
 	queue_free_space_update (RB_MTP_SOURCE (target));
 	return FALSE;
 }
