@@ -518,6 +518,15 @@ rb_playlist_manager_is_dirty (RBPlaylistManager *mgr)
 				(GtkTreeModelForeachFunc) _is_dirty_playlist,
 				&dirty);
 
+	/* explicitly check the play queue */
+	if (dirty == FALSE) {
+		RBSource *queue_source;
+
+		g_object_get (mgr->priv->shell, "queue-source", &queue_source, NULL);
+		g_object_get (queue_source, "dirty", &dirty, NULL);
+		g_object_unref (queue_source);
+	}
+
 	if (!dirty)
 		dirty = g_atomic_int_get (&mgr->priv->dirty);
 
@@ -856,27 +865,40 @@ rb_playlist_manager_set_automatic_playlist (RBPlaylistManager *mgr,
 	rhythmdb_query_free (query);
 }
 
+static void
+new_automatic_playlist_response_cb (GtkDialog *dialog, int response, RBPlaylistManager *mgr)
+{
+	RBSource *playlist;
+
+	switch (response) {
+	case GTK_RESPONSE_NONE:
+	case GTK_RESPONSE_CLOSE:
+		break;
+
+	default:
+		playlist = rb_playlist_manager_new_playlist (mgr, _("New Playlist"), TRUE);
+
+		rb_playlist_manager_set_automatic_playlist (mgr,
+							    RB_AUTO_PLAYLIST_SOURCE (playlist),
+							    RB_QUERY_CREATOR (dialog));
+		rb_playlist_manager_set_dirty (mgr, TRUE);
+		break;
+	}
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
 static void
 rb_playlist_manager_cmd_new_automatic_playlist (GtkAction *action,
 						RBPlaylistManager *mgr)
 {
-	RBQueryCreator *creator = RB_QUERY_CREATOR (rb_query_creator_new (mgr->priv->db));
-	RBSource *playlist;
+	GtkWidget *creator = rb_query_creator_new (mgr->priv->db);
+	gtk_widget_show_all (creator);
 
-	switch (gtk_dialog_run (GTK_DIALOG (creator))) {
-	case GTK_RESPONSE_NONE:
-	case GTK_RESPONSE_CLOSE:
-		gtk_widget_destroy (GTK_WIDGET (creator));
-		return;
-	}
-
-	playlist = rb_playlist_manager_new_playlist (mgr, _("New Playlist"), TRUE);
-
-	rb_playlist_manager_set_automatic_playlist (mgr, RB_AUTO_PLAYLIST_SOURCE (playlist), creator);
-
-	rb_playlist_manager_set_dirty (mgr, TRUE);
-
-	gtk_widget_destroy (GTK_WIDGET (creator));
+	g_signal_connect (creator,
+			  "response",
+			  G_CALLBACK (new_automatic_playlist_response_cb),
+			  mgr);
 }
 
 typedef struct {
